@@ -1989,6 +1989,8 @@ pkg_jobs_handle_install(struct pkg_solved *ps, struct pkg_jobs *j)
 		flags |= PKG_ADD_NOSCRIPT;
 	if ((j->flags & PKG_FLAG_FORCE_MISSING) == PKG_FLAG_FORCE_MISSING)
 		flags |= PKG_ADD_FORCE_MISSING;
+	if ((j->flags & PKG_FLAG_REGISTER_ONLY) == PKG_FLAG_REGISTER_ONLY)
+		flags |= PKG_ADD_REGISTER_ONLY;
 	if (ps->type != PKG_SOLVED_INSTALL) {
 		flags |= PKG_ADD_UPGRADE;
 		if (ps->type == PKG_SOLVED_UPGRADE_INSTALL)
@@ -2018,6 +2020,8 @@ pkg_jobs_handle_delete(struct pkg_solved *ps, struct pkg_jobs *j)
 	flags = 0;
 	if ((j->flags & PKG_FLAG_NOSCRIPT) != 0)
 		flags |= PKG_DELETE_NOSCRIPT;
+	if ((j->flags & PKG_FLAG_KEEPFILES) != 0)
+		flags |= PKG_DELETE_KEEPFILES;
 	if (ps->type == PKG_SOLVED_UPGRADE_REMOVE) {
 		flags |= PKG_DELETE_UPGRADE;
 		rpkg = ps->xlink->items[0]->pkg;
