@@ -27,7 +27,6 @@ int default_yes; /* Default always yes */
 int yes; /* Assume always yes */
 int dry_run; /* Do not perform any actions */
 bool auto_update; /* Do not update repo */
-int case_sensitive; /* Case sensitive queries */
 int force; /* Forced operation */
 int quiet; /* Silent output */
 int newpkgversion; /* New package version is available */
@@ -40,7 +39,6 @@ set_globals(void)
 	yes = pkg_object_bool(pkg_config_get("ASSUME_ALWAYS_YES"));
 	dry_run = 0;
 	auto_update = pkg_object_bool(pkg_config_get("REPO_AUTOUPDATE"));
-	case_sensitive = pkg_object_bool(pkg_config_get("CASE_SENSITIVE_MATCH"));
 	force = 0;
 	quiet = 0;
 	newpkgversion = 0;
